# Generated from age.g4 by ANTLR 4.9.2
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .ageParser import ageParser
else:
    from ageParser import ageParser

# This class defines a complete listener for a parse tree produced by ageParser.
class ageListener(ParseTreeListener):

    # Enter a parse tree produced by ageParser#ageout.
    def enterAgeout(self, ctx:ageParser.AgeoutContext):
        pass

    # Exit a parse tree produced by ageParser#ageout.
    def exitAgeout(self, ctx:ageParser.AgeoutContext):
        pass


    # Enter a parse tree produced by ageParser#vertex.
    def enterVertex(self, ctx:ageParser.VertexContext):
        pass

    # Exit a parse tree produced by ageParser#vertex.
    def exitVertex(self, ctx:ageParser.VertexContext):
        pass


    # Enter a parse tree produced by ageParser#edge.
    def enterEdge(self, ctx:ageParser.EdgeContext):
        pass

    # Exit a parse tree produced by ageParser#edge.
    def exitEdge(self, ctx:ageParser.EdgeContext):
        pass


    # Enter a parse tree produced by ageParser#path.
    def enterPath(self, ctx:ageParser.PathContext):
        pass

    # Exit a parse tree produced by ageParser#path.
    def exitPath(self, ctx:ageParser.PathContext):
        pass


    # Enter a parse tree produced by ageParser#value.
    def enterValue(self, ctx:ageParser.ValueContext):
        pass

    # Exit a parse tree produced by ageParser#value.
    def exitValue(self, ctx:ageParser.ValueContext):
        pass


    # Enter a parse tree produced by ageParser#properties.
    def enterProperties(self, ctx:ageParser.PropertiesContext):
        pass

    # Exit a parse tree produced by ageParser#properties.
    def exitProperties(self, ctx:ageParser.PropertiesContext):
        pass


    # Enter a parse tree produced by ageParser#pair.
    def enterPair(self, ctx:ageParser.PairContext):
        pass

    # Exit a parse tree produced by ageParser#pair.
    def exitPair(self, ctx:ageParser.PairContext):
        pass


    # Enter a parse tree produced by ageParser#arr.
    def enterArr(self, ctx:ageParser.ArrContext):
        pass

    # Exit a parse tree produced by ageParser#arr.
    def exitArr(self, ctx:ageParser.ArrContext):
        pass



del ageParser